// UrsWiFi
//
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-UrsWiFi.html
// Created: 2017-04-27
//
// Version 1.0 (2017-01-11)
// - Basis-Version
//
// Version 1.1 (2017-05-30)
// - Zweite Version von begin mit Strings als Parameter
//
// Version 1.2 (2018-01-29)
// - getLimitedBroadcastAddress() und getDirectedBroadcastAddress hinzugefgt
// - Meldungen in deutsch
//
// Version 1.3 (2018-03-04)
// - Die Einstellung des Modus wird der Methode WiFi.begin berlassen
// - Lokale IP-Adresse kann eingestellt werden
// - Kein Abbruch, wenn der Hostname nicht gesetzt werden kann
// - Returncode gendert
//
// Version 1.4 (2018-03-08)
// - Probleme beim setzen des hostname. Der disassemblierte Quellcode von
//   wifi_station_set_hostname zeigt eine Abfrage auf wifi_get_opmode.
//   Es scheint, dass WiFi.hostname(char *) nur dann funktioniert, wenn
//   der Station-Modus aktiv ist.
//   WiFi.enableSTA(true); eingefgt.
//
// Version 1.5 (2018-03-12)
// - handleReconnectStation() und setRetryInterval() hinzugefgt
//
// Version 1.6 (2018-03-15)
// - getModeString() und getStatusString() hinzugefgt
//
// Version 1.7 (2018-04-09)
// - handleReconnectStation() und setRetryInterval() wieder entfernt.
//   Mit version 2.4.1 Core-Version klappt autoreconnect (wieder?).
//
// Version 1.8 (2018-07-11)
// - Timeout (einstellbar) und Fehlermeldungen fr connectStation berarbeitet.

#ifndef _URSWIFI_h
#define _URSWIFI_h

#include "arduino.h"
#include <ESP8266WiFi.h>

namespace UrsCodes {
  enum ConnectCode : uint8_t { // Flags
    noError = 0,      // Funktion erfolgreich ausgefhrt
    notConnected = 1, // Verbindung konnte nicht hergestellt werden
    hostnameFail = 2, // Hostname konnte nicht eingestellt werden
    localIPFail = 4,  // Lokale IP-Adresse konnte nicht eingestellt werden
    timeout = 8       // Timeout beim Verbinden mit dem WLAN 
  };
}

class UrsWiFiClass {
protected:
  static uint32_t timeOut; // Timeout in Millisekunden, Default ist 30 Sekunden (30.000);

public:
  // Verbindung zum WLAN aufbauen
  // Ein evtl. eingerichteter Access-Point (softAP) wird nicht verndert
  static UrsCodes::ConnectCode connectStation(const char* ssid, const char* password, const char* hostname = NULL, IPAddress localIP = (uint32_t) 0, Print& outDevice = Serial);
  static UrsCodes::ConnectCode connectStation(const String ssid, const String password, const String hostname = "", IPAddress localIP = (uint32_t)0, Print& outDevice = Serial);

  // Liefern die IP-Broadcast-Adressen fr das lokale Netz
  // https://de.wikipedia.org/wiki/Broadcast#IP-Broadcasts
  static IPAddress getLimitedBroadcastAddress(); // Liefert die IP-Adresse 255.255.255.255.
  static IPAddress getDirectedBroadcastAddress(); // 255 an den Stellen mit der Subnet-Mask-Stelle = 0

  // Liefert den Text, z.B. "WIFI_OFF" (s. WiFiMode_t), zum angegebenen Modus.
  // Ist der Parameter -1, wird der aktuelle Modus per WiFi.getMode() abgerufen.
  // Entspricht der Parameter keinem gltigen Modus, wird "WIFI_UNKNOWN" zurck gegeben. 
  static String getModeString(int mode = -1);

  // Liefert den Text, z.B. "WL_CONNECTED" (s. wl_status_t), zum angegebenen Status.
  // Ist der Parameter -1, wird der aktuelle Status per WiFi.status() abgerufen.
  // Entspricht der Parameter keinem gltigen Status, wird "WL_UNKNOWN" zurck gegeben. 
  static String getStatusString(int status = -1);

  // Legt die Zeitbeschrnkung fr den Verbindungsaufbau fest.
  // Default ist 30 Sekunden.
  // 0 bedeutet endlos.
  static void setTimeOut(uint32_t ms) { timeOut = ms; }
};

extern UrsWiFiClass UrsWiFi;
#endif

